<?php
/*
    Plugin Name: Brandsoft Team Viewer
    Description: Brandsoft Team Viewer allows you to showcase your team(s) on your wordpress site
    Version: 1.0.0
    Author: Brandsoft Infotech Pvt. Ltd.
    Author URI: http://www.brandsoftonline.com/
*/

// If Direct Access
if ( !defined( 'ABSPATH' ) ) {

    exit;

}

if ( !class_exists( 'BrandsoftTeam' ) ) {

    function bsteam_activation() {

        BrandsoftTeam::get_instance();
        
    }

    //Utility Function to sanitize input
    function bstv_strip_tags_and_slashes( $str ) {

        return stripslashes( strip_tags( $str ) );

    }

    // Activation Hook
    add_action( 'plugins_loaded', 'bsteam_activation' );

    class BrandsoftTeam {

        private static $instance = NULL;
        private $settings;

        public static function get_instance() {

            if ( NULL == self::$instance ) {

                self::$instance = new self;

            }
            return self::$instance;

        }

        public function __construct() {

            $this->settings = array(

                'bstv_plugin_path'   =>  plugin_dir_path( __FILE__ ),
                'bstv_plugin_url'    =>  plugin_dir_url( __FILE__ ),
                'bstv_plugin_base'   =>  dirname( plugin_basename( __FILE__ ) ),
                'bstv_plugin_file'   =>  __FILE__,

            );
            $this->configure();
            $this->admin();

        }

        public function configure() {

            // Hooks
            add_action( 'wp_enqueue_scripts', array( $this, 'script_style' ) ); // To include JS & CSS
            add_action( 'init', array( $this, 'create_member' ) ); // To set Team & Team Member options under bsteam
            add_action( 'init', array( $this, 'image_size' ) ); // To customize image size
            add_action( 'wp_head',  array( $this, 'css' ) );    //
            add_shortcode( 'bsteam', array( $this, 'bsteam_shortcode' ) ); // To create Shortcode for each team
            register_uninstall_hook( __FILE__, 'BrandsoftTeam::bsteam_uninstall_plugin' );

        }

        public function script_style() {

            // Include CSS
            wp_enqueue_style( 'team', plugins_url( 'css/bsteam-style.css', $this->settings[ 'bstv_plugin_file' ] ), FALSE, 'all' );

        }

        public function create_member() {

            // Create bsteam_member post type
            if ( post_type_exists( "bs_team_member" ) ) {

                return;

            }
            $singular = __( 'Team Member', 'team' );
            $plural   = __( 'Team Members', 'team' );
            $labels = array(

                'name' => $plural,
                'singular_name' => $singular,
                'menu_name' => __( 'BSTeam', 'team' ),
                'add_new' => __( 'Add New Member', 'team' ),
                'add_new_item' => sprintf( __( 'Add %s', 'team' ), $singular ),
                'new_item' => sprintf( __( 'New %s', 'team' ), $singular ),
                'edit_item' => sprintf( __( 'Edit %s', 'team' ), $singular ),
                'view_item' => sprintf( __( 'View %s', 'team' ), $singular ),
                'all_items' => sprintf( __( 'Members', 'team' ) ),
                'search_items' => sprintf( __( 'Search %s', 'team' ), $plural ),
                'not_found' => sprintf( __( 'No %s found', 'team' ), $plural ),
                'not_found_in_trash' => sprintf( __( 'No %s found in trash', 'team' ), $plural ),

            );
            $member_args = array(

                'labels' => $labels,
                'description' => sprintf( __( 'This is where you can create and manage %s.', 'team' ), $plural ),
                'publicly_queryable' => FALSE,
                'show_ui' => TRUE,
                'show_in_menu' => TRUE,
                'capability_type' => 'post',
                'supports' => array( 'title', 'editor', 'thumbnail', ),
                'menu_icon' => 'dashicons-admin-users',

            );
            register_post_type( 'bs_team_member', $member_args );

            // Create bsteam_team post type
            if ( post_type_exists( "bs_team" ) ) {

                return;

            }
            $singular = __( 'Team', 'team' );
            $plural   = __( 'Teams', 'team' );
            $labels = array(

                'name' => $plural,
                'singular_name' => $singular,
                'menu_name' => __( 'BSTeam Team', 'team' ),
                'add_new' => __( 'Add New Team', 'team' ),
                'add_new_item' => sprintf( __( 'Add %s', 'team' ), $singular ),
                'new_item' => sprintf( __( 'New %s', 'team' ), $singular  ),
                'edit_item' => sprintf( __( 'Edit %s', 'team' ), $singular ),
                'view_item' => sprintf( __( 'View %s', 'team' ), $singular ),
                'all_items' => sprintf( __( 'Teams', 'team' ) ),
                'search_items' => sprintf( __( 'Search %s', 'team' ), $plural ),
                'not_found' => sprintf( __( 'No %s found', 'team' ), $plural ),
                'not_found_in_trash' => sprintf( __( 'No %s found in trash', 'team' ), $plural ),

            );
            $team_args = array(

                'labels' => $labels,
                'description' => sprintf( __( 'This is where you can create and manage %s.', 'team' ), $plural ),
                'show_ui' => TRUE,
                "show_in_menu" => 'edit.php?post_type=bs_team_member',
                'capability_type' => 'post',
                'supports' => array( 'title', ),

            );
            register_post_type( 'bs_team', $team_args );

        }

        public function admin() {

            if ( is_admin() ) {

                // Admin Panel Hooks
                // Action Section
                add_action( 'add_meta_boxes', array( $this, 'load_metabox' ) );
                add_action( 'save_post', array( $this, 'save_mb_data' ), 10, 3 );
                add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ) );
                add_action( 'admin_menu', array( $this, 'set_submenu' ), 12 );
                add_action( 'manage_bs_team_member_posts_custom_column' , array( $this, 'member_data_column' ) , 10, 2 );
                add_action( 'manage_bs_team_posts_custom_column' , array( $this, 'team_data_column' ) , 10, 2 );
                add_action( 'edit_form_after_title', array( $this, 'preview' ) );

                // Filter Section
                add_filter( 'manage_bs_team_member_posts_columns', array( $this, 'member_column' ) );
                add_filter( 'manage_bs_team_posts_columns', array( $this, 'team_column' ) );
                add_filter( 'admin_post_thumbnail_html', array( $this, 'image' ) );
                add_filter( 'admin_post_thumbnail_size', array( $this, 'size' ) );
                add_filter( 'enter_title_here', array( $this, 'bs_team_member_title' ) );

            }

        }

        public function load_metabox() {

            add_meta_box( 'member_info', __( 'Team Member Information', 'team' ), array( $this, 'team_member_info' ), 'bs_team_member' );
            add_meta_box( 'team_info', __( 'Team Information', 'team' ), array( $this, 'bsteam_team_info' ), 'bs_team', 'normal', 'high' );

        }

        public function team_member_info( $post ) {

            wp_nonce_field( basename( __FILE__ ), 'bsteam_metabox_info' );
            $contact_info = get_post_meta( $post->ID, 'contact_info', TRUE );
            $social_info  = get_post_meta( $post->ID, 'social_info', TRUE );
            $icon  = array( 'Linkedin', 'Facebook', 'GitHub', 'YouTube', 'Twitter', 'Stackoverflow' );

            $args_all_teams = array( 'post_type' => 'bs_team', 'posts_per_page' => -1, );
            $all_teams = new WP_Query( $args_all_teams );

            //query arguments to retrieve all the teams that the team member represented by $post belongs to
            $args_member_teams = array(
                'post_type' => 'bs_team',
                'meta_query' => array(
                    array(
                        'key' => 'memberlist',
                        'value' => ':' . $post->ID . ';',
                        'compare' => 'LIKE',
                    )
                )
            );

            $member_teams = new WP_Query( $args_member_teams );
            $member_team_ids = array();

            if ( $member_teams->have_posts() ) {

                $member_team_ids = wp_list_pluck( $member_teams->posts, 'ID' );

            }

            include $this->settings[ 'bstv_plugin_path' ] . 'includes/bsteam-member-info.php';

        }

        public function select( $name, $options, $selected = "", $selecttext = "", $class = "", $optionvalue = 'value' ) {

            if ( is_array( $options ) ) {

                $select_html = "<select name=\"$name\" id=\"$name\" class=\"$class\">";
                if ( $selecttext ) {

                    $select_html .= '<option value="">' . $selecttext . '</option>';

                }

                foreach ( $options as $key => $option ) {

                    if ( $optionvalue == 'value' ) {

                        $value = $option;

                    } else {

                        $value = $key;

                    }

                    $select_html .= "<option value=\"$value\"";
                    if ( $value == $selected ) {

                        $select_html .= ' selected="selected"';

                    }
                    $select_html .= ">$option</option>\n";

                }
                $select_html .= '</select>';
                echo $select_html;

            }

        }

        public function bsteam_team_info( $post ) {

            wp_nonce_field( basename( __FILE__ ), 'bsteam_metabox_info' );
            $args = array( 'post_type' => 'bs_team_member', 'posts_per_page' => -1, );
            $members = new WP_Query( $args );
            $options = $this->view_option( 'bs_team', $post->ID );
            $defaultimage = $this->settings[ 'bstv_plugin_url' ] . 'images/default.png';

            include $this->settings[ 'bstv_plugin_path' ] . 'includes/bsteam-team-info.php';

        }

        public function bsteam_view_style( $options, $custom = "" ) {

            $style = array( $options[ 'team-style' ] . '-style', $options[ 'preset' ], 'grid-' . $options[ 'columns' ] . '-col', $custom, 'font-color-' . $options[ 'font_color' ], 'back-color-' . $options[ 'back_color' ], 'shape-' . $options[ 'shape' ], );
            return implode( ' ', $style );

        }

        public function view_option( $postype, $post_id ) {

            $post = get_post( $post_id );

            if ( !$post ) {

                return FALSE;

            }

            $metakeys[ 'bs_team_member' ] = array( 'contact_info', 'social_info', 'bsteam-designation', );
            $metakeys[ 'bs_team' ] = array( 'memberlist', 'team-style', 'preset', 'columns', 'custom_css', 'font_color', 'back_color', 'shape', );
            $options[ 'bs_team_member' ] = array( 'contact_info' => array(), 'social_info' => array(), 'bsteam-designation' => '', );
            $options[ 'bs_team' ] = array( 'memberlist' => array(), 'team-style' => 'cards', 'preset' => '', 'columns' => '', 'custom_css' => '', 'font_color' => '', 'back_color' => '', 'shape' => '', );

            foreach ( $metakeys[ $postype ] as $key => $value ) {

                $metavalue = get_post_meta( $post_id, $value, TRUE );

                if ( $metavalue ) {

                    $options[ $postype ][ $value ] = $metavalue;

                }

            }

            return $options[ $postype ];

        }

        public function save_mb_data( $post_id, $post ) {

            if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {

               return;

            }

            if ( !isset( $_POST[ 'bsteam_metabox_info' ] ) || !wp_verify_nonce( $_POST[ 'bsteam_metabox_info' ], basename( __FILE__ ) ) ) {

                return $post_id;

            }

            $post_type = get_post_type_object($post->post_type);

            if ( !current_user_can( $post_type->cap->edit_post, $post_id ) ) {

                return $post_id;

            }

            if ( $post->post_type == 'bs_team_member' ) {

                $team_repeater = array(

                    'contact_info' => array( 'details' => 'team-content', ),
                    'social_info' => array( 'gateway' => 'team-icon', 'contact_link' => 'team-link', ),

                );
                $team_meta = array( 'bsteam-designation' );

                foreach ( $team_repeater as $key => $value ) {

                    $olddata = get_post_meta( $post_id, $key, TRUE );
                    $newdata = $item = array();

                    foreach ( $value as $k => $v ) {

                        $item[ $k ] = $_POST[ $v ];

                    }
                    $count = count( reset( $item ) );

                    for ( $i = 0; $i < $count; $i++ ) {

                        foreach ( $value as $k => $v ) {

                            if ( $item[ $k ][ $i ] != '' ) {

                                $newdata[ $i ][ $k ] = stripslashes( strip_tags( $item[ $k ][ $i ] ) );

                            }

                        }

                    }

                    if ( !empty( $newdata ) && $newdata != $olddata ) {

                        update_post_meta( $post_id, $key, $newdata );

                    } elseif ( empty( $newdata ) && $olddata ) {

                        delete_post_meta( $post_id, $key, $olddata );

                    }

                }

                $team_list = array();

                if ( isset( $_POST[ 'memberlist' ] ) && is_array( $_POST[ 'memberlist' ] ) ) {

                    $team_list = array_map( 'sanitize_key',  $_POST[ 'memberlist' ]);
                
                }

                $teams_to_remove = array();
                $teams_to_add = array();

                if ( isset( $_POST[ 'old_team_list' ] ) ) {

                    $old_team_str = sanitize_text_field( $_POST[ 'old_team_list' ] );
                    $old_team_arr = explode( ",", $old_team_str );
                
                }


                $teams_to_remove = array_diff( $old_team_arr, $team_list );
                $teams_to_add = array_diff( $team_list, $old_team_arr );

                foreach ( $teams_to_remove as $team_id ) {

                    $members = get_post_meta( $team_id, 'memberlist', true );

                    if ( $members && in_array( $post_id, $members ) ) {

                        $members = array_diff( $members, array( $post_id ) );
                        update_post_meta( $team_id, 'memberlist', $members );

                    }

                }

                foreach ( $teams_to_add as $team_id ) {

                    $members = get_post_meta( $team_id, 'memberlist', true );

                    if ( $members && !in_array( $post_id, $members ) ) {

                        $members[] = $post_id;

                    } else {

                        $members = array();
                        $members[] = $post_id;  // Adding the team member to the team memberlist metadata item

                    }
                    update_post_meta( $team_id, 'memberlist', $members );

                }

            } elseif ( $post->post_type == 'bs_team' ) {

                $team_meta = array( 'memberlist', 'team-style', 'preset', 'columns', 'custom_css', 'font_color', 'back_color', 'shape', );

            }

            foreach ( $team_meta as $meta_key ) {

                $olddata = get_post_meta( $post_id, $meta_key, TRUE );
                $newdata = array();

                if ( isset( $_POST[ $meta_key ] ) ) {

                    if ( is_array( $_POST[ $meta_key ] ) ) {

                        $newdata = array_map( 'bstv_strip_tags_and_slashes', $_POST[ $meta_key ] );

                    } else {

                        $newdata = bstv_strip_tags_and_slashes( $_POST[ $meta_key ] );

                    }

                    if ( !empty( $newdata ) && $newdata != $olddata ) {

                        update_post_meta( $post_id, $meta_key, $newdata );

                    } elseif ( empty( $newdata ) && $olddata ) {

                        delete_post_meta( $post_id, $meta_key, $olddata );

                    }

                } else {

                    delete_post_meta( $post_id, $meta_key, $olddata );

                }

            }

        }



        public function admin_scripts() {

            global $pagenow, $typenow, $post;

            if ( empty( $typenow ) && !empty( $_GET[ 'post' ] ) ) {

                $post = get_post( $_GET[ 'post' ] );
                $typenow = $post->post_type;

            }

            if ( ( $pagenow == 'post-new.php' or $pagenow == 'post.php' ) and ( $typenow == 'bs_team_member' or $typenow == 'bs_team' ) ) {

                // Include JS
                wp_enqueue_script( 'team-meta-box', plugins_url( 'js/bsteam-admin-sheet.js', $this->settings[ 'bstv_plugin_file' ] ), array( 'jquery', 'jquery-ui-sortable', 'wp-util' ) );
                wp_enqueue_script( 'select_option', plugins_url( 'js/select-sheet.js', $this->settings[ 'bstv_plugin_file' ] ), array( 'jquery' ) );

                // Include CSS
                wp_enqueue_style( 'bsteam-admin', plugins_url( 'css/bsteam-admin-style.css', $this->settings[ 'bstv_plugin_file' ] ), FALSE, 'all' );
                wp_enqueue_style( 'select_option', plugins_url( 'css/select-style.css', $this->settings[ 'bstv_plugin_file' ] ), FALSE, 'all' );
                wp_enqueue_style( 'icomoon-css', plugins_url( 'css/icomoon.css', $this->settings[ 'bstv_plugin_file' ] ), FALSE, 'all' );

            }

        }


        public function set_submenu() {

            add_submenu_page( 'edit.php?post_type=bs_team_member', __( 'Add New Team', 'team' ), __( 'Add New Team', 'team' ), 'manage_options', 'post-new.php?post_type=bs_team' );

        }

        function css() {

            global $wp_query;
            $posts = $wp_query->posts;
            $pattern  = '(bsteam)(?![\w-])([^\]\/]*(?:\/(?!\])[^\]\/]*)*?)(?:(\/)\]|\](?:([^\[]*+(?:\[(?!\/\2\])[^\[]*+)*+)\[\/\2\])?)(\]?)';
            $snippet = '';
            $shortcodes = array();

            if ( $posts ) {

                foreach ( $posts as $post ) {

                    $pattern = get_shortcode_regex();
                    preg_match( '/\[bsteam (.+?)\]/', $post->post_content, $matches );

                    if ( is_array( $matches ) && isset( $matches[ 1 ] ) ) {

                        if ( is_array( shortcode_parse_atts( $matches[ 1 ] ) ) ) {

                             $shortcodes[] = shortcode_parse_atts( $matches[ 1 ] );

                        }

                    }

                    if ( !empty( $shortcodes ) ) {

                        foreach ( $shortcodes as $shortcode ) {

                            if ( isset( $shortcode[ 'id' ] ) ) {

                                $custom_css = get_post_meta( $shortcode[ 'id' ], 'custom_css', TRUE );

                                if ( $custom_css ) {

                                    $snippet .= $custom_css;

                                }

                            }

                        }

                    }

                }

                if ( $snippet ) {

                    printf( '<style type="text/css">%s</style>', $snippet );

                }

            }

        }

        function member_data_column( $column,$post_ID ) {

            $options = $this->view_option( 'bs_team_member', $post_ID );

            switch ( $column ) {

                case 'image':
                    echo the_post_thumbnail( 'thumbnail' );
                    break;
                case 'designation':
                    echo $options[ 'bsteam-designation' ];
                    break;

            }

        }

        function member_column( $columns ) {

            $columns = array( 'cb' => '<input type="checkbox" />', 'title' => __( 'Member Name', 'team' ), 'image' => __( 'Member Photo', 'team' ), 'designation' => __( 'Member Designation', 'team' ), );
            return $columns;

        }

        function team_column( $columns ) {

            $columns = array( 'cb' => '<input type="checkbox" />', 'title' => __( 'Team Name', 'team' ), 'members' => __( 'Team Members', 'team' ), 'shortcode' => __( 'Team Shortcode Definition', 'team' ), );
            return $columns;

        }

        function team_data_column( $column, $post_ID ) {

            $options = $this->view_option( 'bs_team', $post_ID );
            $post = get_post( $post_ID );

            switch ( $column ) {

                case 'members':
                    echo count( $options[ 'memberlist' ] );
                    break;
                case 'preset':
                    echo $options[ 'team-style' ];
                    break;
                case 'shortcode':
                    printf( '[bsteam id="%s"]', $post_ID );
                    break;

            }

        }

        function image( $content ) {

            global $post_type,$post;

            if ( $post_type == 'bs_team_member' ) {

                if ( !has_post_thumbnail( $post->ID ) ) {

                   $content .= '<p>' . __( 'Please upload photo (500px X 500px)', 'team' ) . '</p>';

                }

            }
            return $content;

        }

        function size( $thumb_size ) {

            global $post_type,$post;

            if ( $post_type == 'bs_team_member' ) {

                $thumb_size = "bs_team";

            }
            return $thumb_size;

        }

        public function bsteam_view( $team_id, $thumbnail = "bs_team" ) {

            $defaultimage = $this->settings[ 'bstv_plugin_url' ] . 'images/default.png';
            $member_image = get_post_thumbnail_id( $team_id );

            if ( $member_image ) {

                $member_image_url = wp_get_attachment_image_src( $member_image, $thumbnail, TRUE );
                $member_image_url = $member_image_url[ 0 ];

            } else {

                $member_image_url = $defaultimage;

            }
            return $member_image_url;

        }

        public function bsteam_shortcode( $atts ) {

            extract( shortcode_atts( array( 'id' => FALSE ), $atts ) );
            $options = $this->view_option( 'bs_team', $id );

            if ( !$options ) {

                return '<div class="bstv-team-error">' . __( 'No Team Found', 'team' ) . '</div>';

            }

            if ( empty( $options[ 'memberlist' ] ) ) {

                return '<div class="bstv-team-error">' . __( 'No Team Members Found', 'team' ) . '</div>';

            }

            if ( file_exists( $this->settings[ 'bstv_plugin_path' ] . 'includes/view.php' ) ) {

                ob_start();
                $teamargs = array( 'orderby' => 'post__in', 'post_type' => 'bs_team_member', 'post__in' => $options[ 'memberlist' ], 'posts_per_page' => -1, );
                $team = new WP_Query( $teamargs );

                // Include External Setup
                include $this->settings[ 'bstv_plugin_path' ] . 'includes/view.php';
                wp_reset_postdata();
                return ob_get_clean();

            }

        }

        public function bsteam_class( $class ) {

            return implode( ' ', $class );

        }

        public function bsteam_id( $id ) {

            return implode( '-', $id );

        }

        public function bsteam_check( $template, $value, $return = FALSE ) {

            if ( $value ) {

                if ( $return ) {

                    return sprintf( $template, $value );

                } else {

                    echo sprintf( $template, $value );

                }

            }

        }

        public function image_size() {

            if ( function_exists( 'add_image_size' ) ) {

                add_image_size( 'bs_team', 500, 500, TRUE );

            }

        }

        public function preview( $post ) {

            if ( 'bs_team' == $post->post_type && 'publish' == $post->post_status ) {

                printf( '<p>%1$s : [bsteam id="%2$s"]</p>', __( "Use this short code in your page", 'team' ), $post->ID );

            }
            return;

        }

        public function fontcolor( $name, $options, $selected = "", $class = "" ) {

            if ( is_array( $options ) ) {

                $select_html = "<input name=\"$name\" id=\"$name\" class=\"$class\" ";

                if ( $selected == "" ) {

                    $font_value = "";
                    $placeholder = "#ffffff";

                } else {

                    $font_value = $placeholder = $selected;

                }
                $select_html .= "value=\"$font_value\" placeholder=\"$placeholder\">";

                echo $select_html;
                return $font_value;

            }

        }

        public function backcolor( $name, $options, $selected = "", $class = "" ) {

            if ( is_array( $options ) ) {

                $select_html = "<input name=\"$name\" id=\"$name\" class=\"$class\" ";

                if ( $selected == "" ) {

                    $back_value = "";
                    $placeholder = "#25aad5";

                } else {

                    $back_value = $placeholder = $selected;

                }
                $select_html .= "value=\"$back_value\" placeholder=\"$placeholder\">";

                echo $select_html;
                return $back_value;

            }

        }

        public static function bsteam_remove_post_data() {

            // get WP's wpdb class
            global $wpdb;
            $data_removed = false;

            try {

                $table_name = $wpdb->prefix . "posts";
                // set up custom post types array
                $custom_post_types = array( 'bs_team', 'bs_team_member' );
                // remove data from the posts db table where post types are equal to our custom post types
                $data_removed = $wpdb->query(
                    $wpdb->prepare(
                        "
                            DELETE FROM $table_name
                            WHERE post_type = %s OR post_type = %s

                        ",
                        $custom_post_types[0],
                        $custom_post_types[1]
                    )

                );

                $table_name_1 = $wpdb->prefix . "postmeta";
                $table_name_2 = $wpdb->prefix . "posts";

                // delete orphaned meta data
                $wpdb->query(
                    "
                        DELETE pm
                        FROM $table_name_1 pm
                        LEFT JOIN $table_name_2 wp ON wp.ID = pm.post_id
                        WHERE wp.ID IS NULL

                    "
                );

            } catch ( Exception $e ) {
                // php error
            }
            return $data_removed;

        }

        public static function bsteam_uninstall_plugin() {

            // remove custom post types posts and data
            BrandsoftTeam::bsteam_remove_post_data();

        }

        public function bs_team_member_title( $title ) {

            $screen = get_current_screen();

            if ( 'bs_team_member' == $screen->post_type ) {

                $title = "Enter Member Name";

            } else if ( 'bs_team' == $screen->post_type ) {

                $title = "Enter Team Name";

            } else {

                $title = "";

            }
            return $title;

        }

    }

}
